#
#  Class for clothes availability and fit.
#

# Key constants for clothing items.
#
define  ckBra = 'bra'
define  ckJeans = 'jeans'
define  ckSwimsuit = 'swimsuit'
define  ckPanties = 'panties'
define  ckTshirt = 'tshirt'
define  ckWork = 'work'

init python:

    # Very simple clothing system.
    #
    # Records the largest size owned, assuming all smaller sizes are
    # already owned.
    #
    class Clothing:
        sizeMap = dict()            # Map of clothing part to size
        dbgStr = "Empty"            # Text representation

        # Assign the map to use.
        # Must be called before use!
        #
        def setMap(self, map):
            self.sizeMap = map
            self.init()
            if debug:
                dbg("Clothing.setMap({}) {}".format(type(map), self.dbgStr))

        # Set up initial state.
        #
        def init(self):
            self.sizeMap[ckBra] = 2;
            self.sizeMap[ckJeans] = 2;
            self.sizeMap[ckTshirt] = 2;
            self.sizeMap[ckPanties] = 2;
            self.sizeMap[ckWork] = 4;
            self.dbgStr = self.debug()

        # Is an article of the right size owned?
        #
        def has(self, gsize, cKey):
            result = False
            if cKey in self.sizeMap:
                got = self.sizeMap[cKey]
                # if debug:
                    # dbg("Clothing.has: found {} size {}".format(cKey, got))
                result = gsize <= got
            if debugM:
                dbg("Clothing.has({},{})={}".format(gsize, cKey, result))
            return result

        # Are all listed articles of the right size?
        #
        def hasAll(self, gsize, *cKeys):
            for cKey in cKeys:
                if not self.has(gsize, cKey):
                    return False
            return True

        # Is an article of any size owned?
        #
        def any(self, cKey):
            return cKey in self.sizeMap

        # Buy a new size.
        #
        def buy(self, cKey, gsize):
            if cKey in self.sizeMap:
                self.sizeMap[cKey] = max(gsize, self.sizeMap[cKey])
            else:
                self.sizeMap[cKey] = gsize
            self.dbgStr = self.debug()
            if debug:
                dbg("Clothing.buy({},{})".format(cKey, gsize))

        # Lose an article of a given size.
        # Sets the available sizes to gsize -1.
        # If gsize is zero removes the item completely.
        #
        def lose(self, cKey, gsize):
            if cKey in self.sizeMap:
                if gsize == 0:
                    self.sizeMap.pop(cKey)
                else:
                    self.sizeMap[cKey] = gsize - 1
            self.dbgStr = self.debug()
            if debug:
                dbg("Clothing.lose({},{})".format(cKey, gsize))

        # Lose all articles.
        #
        def loseAll(self, cKey):
            self.lose(cKey, 0)

        # List the available sizes.
        #
        def debug(self, group=4):
            count = 1
            total = 1
            result = ""
            for name in sorted(self.sizeMap.keys()):
                result += "{}={}".format(name, self.sizeMap[name])
                if count < group:
                    result += " "
                    count += 1
                    total += 1
                elif total < len(self.sizeMap):
                    result += "\n"
                    count = 0

            # Work around for len(flagSet) failing on rollback
            if result == "":
                result = "Empty"

            return result
