# Generic set of counters.
#
init python:

    class Counters:
        dbgStr = "Unused"
        map = dict()

        # Assign the map to use.
        # Must be called before use!
        #
        def setMap(self, map):
            self.map = map
            self.dbgStr = self.debug()
            if debug:
                dbg("Counters.setMap({}) size={}".format(type(map), len(map)))

        # Get the current value of the named counter.
        # If the counter is not defined return 0
        #
        def get(self, name):
            if name in self.map:
                return self.map[name]
            return 0

        # Increment the named counter.
        # If the counter is not defined it is set to 1
        # Returns the new value.
        #
        def inc(self, name):
            if name in self.map:
                self.map[name] += 1
            else:
                self.map[name] = 1
            self.dbgStr = self.debug()
            return self.map[name]

        # Decrement the named counter.
        # If the counter is not defined it is set to -1
        #
        def dec(self, name):
            if name in self.map:
                self.map[name] -= 1
            else:
                self.map[name] = -1
            self.dbgStr = self.debug()
            return self.map[name]

        # Set a named counter to a value (default 1)
        #
        def set(self, name, value=1):
            self.map[name] = value
            self.dbgStr = self.debug()

        # Set a named counter to zero if it exists.
        #
        def reset(self, name):
            if name in self.map:
                self.map[name] = 0
            self.dbgStr = self.debug()

        # Delete a named counter.
        #
        def delete(self, name):
            if name in self.map:
                del self.map[name]
                self.dbgStr = self.debug()

        # Returns true only if the named counter is set to 2.
        #
        def isDone(self, name):
            result = False
            if name in self.map:
                result = self.map[name] == 2
            return result

        # Returns true if the named count is zero or undefined.
        # Increments the named counter.
        #
        def isFirst(self, name, lim=1):
            result = True
            if name in self.map:
                result = self.map[name] < lim
                self.map[name] += 1
            else:
                self.map[name] = 1
            self.dbgStr = self.debug()
            return result

        # Returns true only if the named counter is set to 1.
        #
        def isOne(self, name):
            result = False
            if name in self.map:
                result = self.map[name] == 1
            return result

        # Returns true only if the named counter is set to 1, and changes it to 2.
        #
        def isOnce(self, name):
            result = False
            if name in self.map:
                value = self.map[name]
                if value == 1:
                    self.map[name] = 2
                    result = True
                    self.dbgStr = self.debug()
            return result

        # Returns true only if the named counter is zero or not set.
        #
        def isZero(self, name):
            result = True
            if name in self.map:
                result = self.map[name] == 0
            return result

        # Clear all entries.
        #
        def clearAll(self):
            self.map.clear()
            self.dbgStr = self.debug()

        # List the set flags.
        #
        def debug(self, group=4):
            count = 1
            result = ""
            for name in sorted(self.map.keys()):
                result += "{}={}".format(name, self.map[name])
                if count < group:
                    result += " "
                    count += 1
                else:
                    result += "\n"
                    count = 0

            # Work around for len(flagSet) failing on rollback
            if result == "":
                result = "Empty"

            return result

# Dictionary to work with.
# Should be a member, but then renpy won't persist it properly.
#default self.map = dict()     # general purpose counters
