# Customisation of food menu..
#
init python:

    # Additional information passed to food menus.
    #
    # If an entry matches the menu item prompt then the entries here replace it
    # in screen foodMenu.
    # When an entry is chosen in a food menu it's prompt is recorded in
    # foodMenuSeln. This can be passed to addMealAcc to keep track of the
    # total meal size.
    #
    class FoodMenu:
        mapM = dict()               # Map of name to FoodItem
        stylePrefixM = "food"       # food menu style prefix
        titleM = None               # Menu title eg. "Bill's Fine Dining"
        subtitleM = None            # Menu subtitle eg. "Dessert Menu"
        mealAcc = 0                 # Meal size accumulator

        # Assign the map to use.
        # Must be called before use!
        #
        def setMap(self, map):
            self.mapM = map
            if debug:
                dbg("FoodMenu.setMap({}) size={}".format(type(map), len(map)))

        # Start a new menu.
        # Optionally assign title and subtitle.
        #
        def start(self, title=None, subtitle=None):
            self.mapM.clear()
            self.titleM = title
            self.subtitleM = subtitle

        # Start a new menu.
        # Optionally assign title and subtitle.
        # Clears the meal size accumulator.
        #
        def startClear(self, title=None, subtitle=None):
            self.mapM.clear()
            self.titleM = title
            self.subtitleM = subtitle
            self.mealAcc = 0

        # Reset the meal size accumulator.
        #
        def resetMealAcc(self):
            self.mealAcc = 0

        # Add to the meal size accumulator.
        # name: the food item name used in the menu
        #
        def addMealAcc(self, name):
            if name in self.mapM:
                self.mealAcc += self.mapM[name].msize
            if debug:
                dbg("FoodMenu.addMealAcc({})={}".format(name, self.mealAcc))

        # Does the PC have space for a dish of a given size?
        #
        # The global variable mealsize must be defined before use.
        #
        def isSpaceFor(self, dsize):
            result = mealsize >= dsize + self.mealAcc
            if debug:
                dbg("FoodMenu.isSpaceFor({})={} mealsize={}".format(dsize, result, mealsize))
            return result

        # Add a food item.
        # name: the food name from FoodDict
        # prompt: the human food name
        # cost: what the pc has spent so far
        # size: the meal size for this item
        # treat: true if this meal is free
        # food: optional food dict key to replace name
        # price: optional price to override the one from food dict
        #
        # The global variables pc, mealsize, and foods must be defined before use.
        #
        def addFood(self, name, prompt, cost=0, msize=0, treat=False, food=None, price=None):
            item = FoodItem()
            item.prompt = prompt
            item.msize = msize
            if not food:
                food = name
            if food in foods:
                if price:
                    item.cost = price
                else:
                    tuple = foods[food]
                    item.cost = tuple[3]
                if msize + self.mealAcc > mealsize:
                    item.available = False
                    item.post = " (too heavy)"
                elif treat:
                    pass
                elif not pc.canAfford(item.cost, cost):
                    item.available = False
                    item.post = " (too expensive)"
                if debugM:
                    dbg("FoodMenu.addFood({},{},{},{},{},{})={}".format(name, prompt, cost, msize, treat, food, item.text()))
            else:
                item.post = " (unknown food ''" + name + "'')"
                dbg("FoodMenu.addFood({},{},{},{},{},{})={}".format(name, prompt, cost, msize, treat, food, item.text()))
            self.mapM[name] = item

        # Add a menu section header.
        # name: the header text
        #
        def addHeader(self, name):
            item = FoodItem()
            item.prompt = name
            item.header = True
            self.mapM[name] = item
            if debugM:
                dbg("FoodMenu.addHeader({})={}".format(name, item.text()))

        # Get the food item corresponding to a name.
        #
        def get(self, name):
            return self.mapM[name]

        # Determine if a name is in the food menu map.
        #
        def has(self, name):
            if name in self.mapM:
                return True
            return False

        # Determine if an item is available to select.
        #
        def isAvailable(self, name):
            return self.mapM[name].available

        # Get the text associated with an item.
        #
        def text(self, name):
            return self.mapM[name].text()

# EoF
