# NPC Character stats
#
init python:

    # Basic character stats.
    #
    class Npc:
        metCount = 0            # Number of times met
        dateCount = 0           # Number of dates
        dateLastDay = 0         # Last date day number
        canCall = False         # True if pc has their number
        romance = 0             # Romance level
        feeder = 0              # Feeder level
        feedee = 0              # Feedee level
        breakup = False         # True if broken up
        breakCount = 0          # Number of breakups
        breakUntil = 0          # On a break until this day
        pcMinBfp = 10           # Minimum player bfp
        pcMaxBfp = 35           # Maximum player bfp
        weight = 50             # Weight in kg
        tipYes = 0              # Tips given
        tipNo = 0               # Tips not given

        # Increase metCount by one.
        #
        def meet(self):
            self.metCount += 1

        # Increase dateCount by one and set dateLastDay.
        #
        def date(self):
            self.dateCount += 1
            self.dateLastDay = gt.dayNum

        # Has the pc met them before?
        #
        def isMet(self):
            return self.metCount >= 1

        # Has the pc met them count times?
        #
        def isMetCount(self, count):
            return self.metCount == count

        # Has the pc met them count times or more?
        #
        def isMetOrMore(self, count):
            return self.metCount >= count

        # Had the pc not met them before?
        #
        def isNotMet(self):
            return self.metCount == 0

        # Is the PC taking a break from the NPC?
        #
        def isOnBreak(self):
            if self.breakUntil == 0:
                if debug:
                    dbg("isOnBreak()=False as time elapsed")
                return False

            result = self.breakUntil > gt.dayNum

            if debug:
                dbg("isOnBreak()={}".format(result))

            return result

        # Is the romance greater or equal to value?
        # If the value is zero, true if romance > 0.
        # During breakup this is always false.
        #
        def isRomance(self, value=0):
            if self.breakup:
                return False
            if value == 0:
                return self.romance > 0

            return self.romance >= value

        # Add to the romance stat.
        # Negative values reduce, but not below zero.
        # Positive values increase, but not beyond 100.
        #
        def addRomance(self, value):
            self.romance += value
            if self.romance < 0:
                self.romance = 0
            if self.romance > 100:
                self.romance = 100
            if debug:
                dbg("npc.addRomance({})={}".format(value, self.romance))

        # Is feedee greater or equal to value?
        # If the value is zero, true if feeder > 0.
        #
        def isFeedee(self, value=0):
            if value == 0:
                return self.feedee > 0

            return self.feedee >= value

        # Is feeder greater or equal to value?
        # If the value is zero, true if feeder > 0.
        #
        def isFeeder(self, value=0):
            if value == 0:
                return self.feeder > 0

            return self.feeder >= value

        # Add to the feedee stat.
        # Negative values reduce, but not below zero.
        #
        def addFeedee(self, value):
            self.feedee += value

            if self.feedee < 0:
                self.feedee = 0
            if self.feedee > 100:
                self.feedee = 100
            if debug:
                dbg("npc.addFeedee({})={}".format(value, self.feedee))

        # Add to the feeder stat.
        # Negative values reduce, but not below zero.
        #
        def addFeeder(self, value):
            self.feeder += value

            if self.feeder < 0:
                self.feeder = 0
            if self.feeder > 100:
                self.feeder = 100
            if debug:
                dbg("npc.addFeeder({})={}".format(value, self.feeder))

        # Determine NPC opinion of pc's weight.
        #
        # 'tooThin' : Much too thin
        # 'thin'    : On the thin side
        # 'good'    : A good weight
        # 'fat'     : Too fat
        # 'tooFat'  : Much too fat
        #
        def opinionPc(self):
            bfp = pc.getBodyFatPer()
            result = "good"

            if bfp < self.pcMinBfp:
                if self.isPcTooThin():
                    result = "tooThin"
                else:
                    result = "thin"

            if bfp > self.pcMaxBfp:
                if self.isPcTooFat():
                    result = "tooFat"
                else:
                    result = "fat"

            if debug:
                dbg("opinionPc() {}".format(result))

            return result

        # Is the pc too thin, taking in to account romance, for this NPC?
        #
        def isPcTooThin(self):
            fact = 1.1 + self.romance/200.0
            fat = pc.fat * fact
            bfp = 100.0 * fat / (fat + pc.base)
            result = bfp < self.pcMinBfp
            # dbg("isPcTooThin()={} fact={} fat={} bfp={} min={}".format(result, fact, fat, bfp, self.pcMinBfp))
            if debug:
                dbg("isPcTooThin()={}".format(result))
            return result

        # Is the PC thin in the npc's opinion?
        #
        def isPcThin(self):
            bfp = pc.getBodyFatPer()
            result = bfp < self.pcMinBfp
            if debugM:
                dbg("isPcThin()={}".format(result))
            return result

        # Is the PC fat in the npc's opinion?
        #
        def isPcFat(self):
            bfp = pc.getBodyFatPer()
            result = bfp > self.pcMaxBfp
            if debugM:
                dbg("isPcFat()={}".format(result))
            return result

        # Is the PC too fat, taking romance into account, in the NPC's opinion?
        #
        # Factor can vary from 0.9 to 0.875 or 11% to 14.3%.
        #
        def isPcTooFat(self):
            fact = 0.9 - self.romance/400.0
            fat = pc.fat * fact
            bfp = 100.0 * fat / (fat + pc.base)
            result = bfp > self.pcMaxBfp
            # dbg("isPcTooFat()={} fact={} fat={} bfp={} max={}".format(result, fact, fat, bfp, self.pcMaxBfp))
            if debug:
                dbg("isPcTooFat()={}".format(result))
            return result

        # Raise max bfp to at least the given value.
        #
        def setMaxBfpAtLeast(self, bfp):
            if self.pcMaxBfp < bfp:
                self.pcMaxBfp = bfp
            if debug:
                dbg("setMaxBfpAtLeast({:0.1f}) max={:0.1f}".format(bfp, self.pcMaxBfp))

        # Set a number of days to break up with this NPC.
        #
        def setBreakDays(self, days):
            if not self.breakup:
                self.breakup = True
                self.breakCount += 1
            self.breakUntil = gt.dayNum + days

        # Get back together.
        #
        def makeup(self):
            self.breakup = False;
            self.breakUntil = 0;

        # Random chance of at least min, but rising to max with max feedee.
        # If the feedee member is zero this test always fails.
        #
        # min   - minimum percentage 0..100
        # max   - maximum percentage 0..100
        #
        def randomFeedee(self, min, max):
            if self.feedee == 0:
                chance = 0
                result = False
            else:
                chance = min + (max - min) * self.feedee / 100.0
                result = pc.randomBool(chance)

            if debugM:
                dbg("randomFeedee({},{})={} chance={}".format(min, max, result, chance))
            return result

        # Random chance of at least min, but rising to max with max feeder.
        # If the feeder member is zero this test always fails.
        #
        # min   - minimum percentage 0..100
        # max   - maximum percentage 0..100
        #
        def randomFeeder(self, min, max):
            if self.feeder == 0:
                chance = 0
                result = False
            else:
                chance = min + (max - min) * self.feeder / 100.0
                result = pc.randomBool(chance)

            if debugM:
                dbg("randomFeeder({},{})={} chance={}".format(min, max, result, chance))
            return result

        # Random chance of at least min, but rising to max with max romance.
        # Always False if on a breakup or romance is zero.
        #
        # min   - minimum percentage 0..100
        # max   - maximum percentage 0..100
        #
        def randomRomance(self, min, max):
            if self.breakup or self.romance == 0:
                chance = 0
                result = False
            else:
                chance = min + (max - min) * self.romance / 100.0
                result = pc.randomBool(chance)

            if debug:
                dbg("randomRomance({},{})={} chance={}".format(min, max, result, chance))
            return result

        # Random chance of at least min, but rising to max with more tips.
        # Always False if fewer than ten tipping oppertunities are recorded.
        #
        # min   - minimum percentage 0..100
        # max   - maximum percentage 0..100
        #
        def randomTip(self, min, max):
            tot = self.tipYes + self.tipNo
            dbg("yes={} no={} tot={}".format(self.tipYes, self.tipNo, tot))
            result = False
            chance = 0
            if tot >= 10:
                chance = min + (max - min) * (self.tipYes * 1.0 / tot)
                result = pc.randomBool(chance)

            if debug:
                dbg("randomTip({},{})={} chance={} tot={}".format(min, max, result, chance, tot))
            return result

        # Record tipping.
        #
        # tipped    - True for tipped, False for not tipped
        #
        def tip(self, tipped):
            if tipped:
                if self.tipYes:
                    self.tipYes += 1
                else:
                    self.tipYes = 1
            else:
                if self.tipNo:
                    self.tipNo += 1
                else:
                    self.tipNo = 1


        def debug(self):
            return "metCount={} canCall={} feeder={} romance={}".format(self.metCount, self.canCall, self.feeder, self.romance)

# EoF
