# Player character class
#
init python:
    import math

    class Player:
        name = "You"            # The pc's name
        drunk = 0               # Alcohol units drunk today
        base = 40               # Base weight in kg
        fat = 0                 # Fat gained in kg
        fatLim = 12             # Limit to have fat in kg the player is willing to get
        full = 0                # Fullness in litres
        greed = 0               # Greediness
        diet = False            # Dieting
        dietDays = 0            # Max days to diet
        dietGoal = 0            # Goal weight to stop
        dietWill = 0            # Dieting willpower
        dietHard = False        # True to enforce dieting
        cheatDay = False        # It's a cheat day now
        dietCount = 0           # Number of diets started
        meals = 0               # Meals eaten today
        gsize = 0               # Graphical size of pc
        money = 5               # Money owned
        energy = 10             # Energy to do things
        mood = 50               # Mood 0..100
        moodStr = "okay"        # Mood name
        weightKg = 40           # Weight in kg
        weightLb = 88           # Weight in lbs
        lastWeightLb = "88?lb"  # Last weight
        lastWeightKg = "40?kg"  # Last weight in kg
        oldKg = 40              # Old weight

        # Set the pc name.
        # Default to Erin.
        #
        def setName(self, name):
            name = name.strip()

            if not name:
                self.name = "Erin"
            else:
                self.name = name.capitalize()

        # Determine what proportion of their capacity the pc will be willing
        # to consume by hour.
        #
        # This is a generally increasing function, starting at 6am, with steps
        # at 12:00 an 18:00. From 18:00 to 05:00 the result is 1.0.
        #
        def getFullFactor(self):
            hour = gt.hour
            result = 1.0
            if hour >=6 and hour < 12:
                result = 0.2 + 0.05 * (hour - 6)
            elif hour >=12 and hour < 18:
                result = 0.6 + 0.05 * (hour - 12)
            if debugM:
                dbg("getFullFactor()={:0.2f} at {}".format(result, hour))
            return result

        # How much to eat in one sitting in plates?
        # This is an approximation of 500cc plates of food.
        #
        # Result is determined by time of day, and what has already been consumed.
        # Greed, drunkeness can increase the result.
        # Mood and confidence can increase or decrease the result.
        #
        # For general prompt generation use changeMood False.
        # For a meal use changeMood True so dieting affects mood.
        # Dieting forces the result to 1 or 0.
        #
        # Min = 0
        # Max = 15(7.5*2) + 3(gf) + 2(df) + 2(cf) + 2(mf) = 24 (12l/26.4lbs)
        #
        def getMealSize(self, changeMood=False):
            # Start with full limit (litres) which depends on fatness
            #
            fullLim = self.getFullLimit()
            fl = fullLim
            reason = "fulLim={:0.1f}".format(fullLim)

            # Use full factor to adjust full limit based on time of day.
            #
            ff = self.getFullFactor()
            fullLim *= ff
            reason += " ff={:0.2f} tgtVol={:0.1f}".format(ff, fullLim)

            # Subtract volume eaten already.
            # This produces a negativve result if overfull.
            #
            result = fullLim - self.full
            reason += " vol={:0.1f}".format(result)

            # Adjust negative volumes towards zero.
            #
            if result < 0:
                result *= 0.25
                reason += ",adjNeg={:0.1f}".format(result)

            # Convert to plates.
            #
            result *= 2
            reason += "/plates={:0.1f}".format(result)

            # Greed can add up to 3 plates.
            #
            gf = 3.0 * self.greed / 100.0
            reason += " greed+{:0.1f}".format(gf)
            result += gf

            # Drunkeness can add up to 2 plates.
            #
            df = 2.0 * min(1.0, self.drunk/12.0)
            reason += " drunk+{:0.1f}".format(df)
            result += df

            # Confidence can add or subtract 2 plates
            #
            cf = 2.0 * max(-1,min(1,((self.fatLim - self.fat) / 50.0)))
            reason += " conf{:+0.1f}".format(cf)
            result += cf

            # Mood can add 2 or sutract 2 plates.
            #
            if self.mood >= 60:
                mf = (self.mood - 60) / 20.0
                reason += " happy{:+0.1f}".format(mf)
            elif self.mood >= 40:
                mf = 0
            elif self.mood >= 20:
                mf = (10 - abs(self.mood - 30)) / 10.0
                reason += " unhappy{:+0.1f}".format(mf)
            else:
                mf = 2 * (20 - self.mood) / 20.0
                reason += " misery{:+0.1f}".format(mf)
            result += mf

            # Capacity can reduce by one fifth.
            #
            cf = (self.full - fl)/fl
            # reason += " fullness={:0.2f}".format(cf)
            if cf > 0.0:
                cf = 1.0 - 0.2 * min(1, cf)
                result *= cf
                reason += " cap*{:0.2f}".format(cf)

            # Diet can limit result to one.
            # If changeMood is true, subtract 1 from mood.
            #
            if self.diet and not self.cheatDay and result > 1:
                result = 1          # eat minimum amount
                reason += " but dieting so 1"
                if changeMood:
                    self.addMood(-1)    # having to diet leads to more unhappiness

            reason += " raw={:0.2f}".format(result)

            # Checks and quantisation.
            #
            if result < 0:
                result = 0
                reason += " neg becomes 0"
            elif result < 0.05:
                result = 0
                reason += " small becomes 0"
            else:
                result = int(round(result))
                if result == 0:
                    reason += " frac becomes 1"
                    result = 1

            if debug:
                dbg("getMealSize({})={} {}".format(changeMood, result, reason))

            return result

        # Get body fat percentage
        #
        def getBodyFatPer(self):
            result = (100 * self.fat)/(self.base + self.fat)

            if debugM:
                dbg("getBodyFatPer() {:0.1f}".format(result))

            return result

        # Get body fat percentage for a given mass
        #
        def getBodyFatPerAt(self, mass):
            fat = mass - self.base
            result = (100 * fat)/mass

            if debug:
                dbg("getBodyFatPerAt({:0.1f}kg/{:0.1f}lb)={:0.1f} ".format(mass, mass * 2.2, result))

            return result

        # Get the body mass for a given bofy fat percentage.
        #
        def getMassForBfp(self, bfp):
            mass = self.base / (1 - bfp/100.0)
            if debug:
                dbg("getMassForBfp({:0.1f})={:0.1f}".format(bfp, mass))
            return mass

        # Update player weight strings after stepping on scales.
        #
        def updateLastWeight(self):
            self.lastWeightLb = "{:0.0f}lb".format(self.weightLb)
            self.lastWeightKg = "{:0.0f}kg".format(self.weightKg)

        # Get the number of days of dieting to reach a goal weight
        #
        def getDaysToDietTo(self, goalKg):
            goalLb = math.floor(goalKg * 2.2)
            loseLb = round(pc.weightLb) - goalLb
            days = int(3 * loseLb)
            if debug:
                dbg("getDaysToDiet({:0.2f}kg)={}".format(goalKg, days))
            return days

        # Get fullness limit in litres.
        #
        # 0:    3
        # 1:    3
        # 2:    3       Typical over consuption
        # 3:    3.5
        # 4:    4
        # 5:    4.5
        # 6:    5
        # 7:    5.5
        # 8:    6
        # 9:    6.5
        # 10:   7
        # 11:   7.5     WR eating contest contender
        #
        def getFullLimit(self):
            if self.gsize < 3:
                result = 3
            elif self.gsize >= 3:
                result = 3 + (self.gsize - 2) / 2

            if debugM:
                dbg("getFullLimit()={}".format(result))

            return result

        # Get fullness as a percentage 0..100++
        #
        def getFullness(self):
            result = self.full * 100 / self.getFullLimit()

            if debugM:
                dbg("getFullness: {}".format(result))

            return result

        # Get remaining capacity, up to full.
        # Returns minCap if result is less than this.
        #
        def getCapacityRemain(self, minCap=0.5):
            result = self.getFullLimit() - self.full
            if result < minCap:
                result = minCap
            if debug:
                dbg("getCapacityRemain({})={:0.2f}".format(minCap, result))
            return result

        # Start a diet.
        # days - maximum duration before abandon
        # goal - target weight to succeed
        #
        def dietStart(self, days, goalKg):
            self.diet = True
            self.dietWill = 90
            self.dietDays = days
            self.dietGoal = goalKg
            self.cheatDay = False
            self.addMood(-20)
            self.oldKg = self.weightKg
            self.dietCount += 1
            if debug:
                dbg("dietStart({}, {:0.1f}kg)".format(days, goalKg))

        # Stop a diet.
        #
        def dietStop(self):
            self.diet = False
            self.dietDays = 0
            self.dietGoal = 0
            self.dietWill = 0
            self.cheatDay = False
            self.addMood(20)

        # Test diet willpower
        # Returns true if still dieting.
        # Returns false and sets cheatDay if check failed.
        # Sets flag dietStopWill if diet ends
        #
        def testDietWill(self):
            if self.cheatDay:
                if debug:
                    dbg("testDietWill()=False cheatDay already")
                return False

            result = self.dietHard or self.randomBool(self.dietWill)

            if not result:
                self.decWill()

            if debug:
                dbg("testDietWill()={} dietHard={} dietWill={} cheatDay={}".format(result, self.dietHard, self.dietWill, self.cheatDay))

            return result

        # Reduce diet willpower and set cheat day.
        # If the pc is hard dieting, does nothing.
        # Sets flag dietStopWill and stops dieting if willpower reaches zero.
        #
        def decWill(self):
            if self.dietHard:
                return
            self.dietWill -= 10
            self.cheatDay = True
            if self.dietWill <= 0:
                self.dietStop()
                flags.set("dietStopWill")

        # Does the pc think they need to diet?
        #
        def isDietConsider(self):
            result = self.fat > self.fatLim

            if debugM:
                dbg("isDietConsider()={}".format(result))

            return result

        # Is the pc restricting their diet?
        # Dieting and not a cheat day.
        #
        def isDietRestricted(self):
            return self.diet and not self.cheatDay

        # Is the pc not dieting or on a cheat day?
        #
        def isDietUnrestricted(self):
            return self.cheatDay or not self.diet

        # Start a new day.
        #
        # May set flags:
        # dietStopGoal if target weight reached
        # dietStopDays if diet had elapsed with time
        #
        def newDay(self):
            self.energy = 10
            self.full = 0
            self.drunk = 0
            self.meals = 0
            self.cheatDay = False
            # self.fat -= 0.32
            fatBurn = bmrFatFem(158, pc.weightKg, 22) # Pc is 5'2", 22yo
            self.fat -= fatBurn

            if debug:
                dbg("pc.newDay(): fat burnt {:0.3f}kg".format(fatBurn))

            if self.fat < 0:
                self.fat = 0
            self.updateWeight()

            if self.diet:
                if self.fat <= self.dietGoal - self.base:
                    self.diet = False
                    self.dietDays = 0
                    flags.set("dietStopGoal")
                    self.addMood(30)

                    if debug:
                        dbg("pc.newDay() diet goal reached")
                elif self.dietHard:
                    if debug:
                        dbg("pc.newDay() still hard dieting")
                else:
                    self.dietDays -= 1

                    if self.dietDays <= 0:
                        self.diet = False
                        self.dietDays = 0
                        flags.set("stopDietDays")
                        self.addMood(20)

                        if debug:
                            dbg("pc.newDay() diet days reached")
                    else:
                        if debug:
                            dbg("pc.newDay() diet ongoing fat={:0.2f} weight={:0.2f} goal={:0.2f}".format(self.fat, self.weightKg, self.dietGoal))

        # Adjective for butt size.
        #
        def adjArse(self):
            if self.gsize == 0:
                return "small"
            if self.gsize == 1:
                return "perky"
            if self.gsize == 2:
                return "round"
            if self.gsize == 3:
                return "plump"
            if self.gsize == 4:
                return "fat"
            if self.gsize == 5:
                return "thick"
            if self.gsize == 6:
                return "ample"
            if self.gsize == 7:
                return "hefty"
            if self.gsize == 8:
                return "weighty"
            if self.gsize == 9:
                return "prodigous"
            if self.gsize == 10:
                return "enormous"
            if self.gsize == 11:
                return "massive"

        # Adjective for belly size.
        #
        def adjBelly(self):
            if self.gsize == 0:
                return "flat"
            if self.gsize == 1:
                return "soft"
            if self.gsize == 2:
                return "round"
            if self.gsize == 3:
                return "plump"
            if self.gsize == 4:
                return "fat"
            if self.gsize == 5:
                return "ample"
            if self.gsize == 6:
                return "heavy"
            if self.gsize == 7:
                return "hefty"
            if self.gsize == 8:
                return "weighty"
            if self.gsize == 9:
                return "prodigous"
            if self.gsize == 10:
                return "enormous"
            if self.gsize == 11:
                return "massive"

        # Adjective for boob size.
        #
        def adjBoob(self):
            if self.gsize == 0:
                return "small"
            if self.gsize == 1:
                return "perky"
            if self.gsize == 2:
                return "round"
            if self.gsize == 3:
                return "plump"
            if self.gsize == 4:
                return "ample"
            if self.gsize == 5:
                return "fat"
            if self.gsize == 6:
                return "heavy"
            if self.gsize == 7:
                return "hefty"
            if self.gsize == 8:
                return "weighty"
            if self.gsize == 9:
                return "prodigous"
            if self.gsize == 10:
                return "enormous"
            if self.gsize == 11:
                return "massive"

        def adjBloat(self):
            full = self.getFullness()

            if full <= 1:
                return "empty"
            if full <= 25:
                return "hungry"
            if full <= 75:
                return "happy"
            if full <= 100:
                return "sated"
            if full <= 150:
                return "gorged"
            if full <= 200:
                return "overladen"
            return "painfully overstuffed"



        # How the player moves, for flavour.
        #
        def locoVerb(self):
            if self.gsize >= 9:
                return "waddle"
            if self.gsize >= 6:
                return "wobble"
            if self.gsize >= 3:
                return "jiggle"
            return "walk"

        # Walk or run a distance.
        #
        def walkRun(self, distKm):
            cal = 110 * distKm
            fat = cal / 7700.0
            self.fat -= fat
            if self.fat < 0:
                self.fat = 0
            self.updateWeight()
            if debug:
                dbg("walkRun({}) use={:0.2f} fat={:0.2f}".format(distKm, fat, self.fat))

        # Excercise.
        #
        # Type      Rate
        # light     300
        # moderate  500
        # hard      700
        # strenous  900
        #
        def exercise(self, rate, hours):
            cal = rate * hours
            fat = cal / 7700.0
            self.fat -= fat
            if self.fat < 0:
                self.fat = 0
            self.updateWeight()
            if debug:
                dbg("exercise({}, {}) used={:0.2f} fat={:0.2f}".format(rate, hours, fat, self.fat))

        # Use energy
        #
        def useEnergy(self, energy):
            self.energy -= energy

            if self.energy < 0:
                self.energy = 0

        # Eat a meal.
        # fat: fat gain in kg
        # full: full gain in litres
        # Increments meal count.
        #
        def eatMeal(self, fat, full):
            self.meals += 1
            self.fat += fat
            self.full += full
            self.updateWeight()
            if debug:
                dbg("eatMeal({}, {}) Meals: {}".format(fat, full, self.meals))

        # Eat a snack.
        # fat: fat gain in kg
        # full: full gain in litres
        # Does not change meal count.
        #
        def eatSnack(self, fat, full):
            self.fat += fat
            self.full += full
            self.updateWeight()
            if debug:
                dbg("eatSnack({}, {})".format(fat, full))

        # Drink alcohol
        # fat: fat gain in kg
        # full: full gain in litres
        # drunk: units drunk
        # Does not change meal count.
        #
        def drink(self, fat, full, drunk):
            self.fat += fat
            self.full += full
            self.drunk += drunk
            self.updateWeight()
            if debug:
                dbg("drink({}, {}, {})".format(fat, full, drunk))

        # Eat food.
        # Clears food variable.
        # (see class Food and food variable)
        def eatFood(self, food):
            self.fat += food.fat
            self.full += food.full
            self.drunk += food.drink

            if food.meal:
                self.meals += 1

            if debug:
                dbg("eatFood({})".format(food.debug(), self.meals))

            food.clear()
            self.updateWeight()

        # Eat food.
        # Does not clears food variable.
        # (see class Food and food variable)
        def eatFoodNc(self, food):
            self.fat += food.fat
            self.full += food.full
            self.drunk += food.drink

            if food.meal:
                self.meals += 1

            if debug:
                dbg("eatFoodNc({})".format(food.debug(), self.meals))

            self.updateWeight()

        # Update total weight
        #
        def updateWeight(self):
            self.weightKg = self.base + self.fat
            self.weightLb = 2.2 * self.weightKg

        # Add to fat limit.
        #
        def addLim(self, confidence):
            self.fatLim += confidence

            if self.fatLim < 5:
                self.fatLim = 5

            if debug:
                dbg("addLim({}) = {:0.2f}".format(confidence, self.fatLim))

        # Add to greed.
        # Limit value to 0..100
        #
        def addGreed(self, greed):
            self.greed += greed

            if self.greed < 0:
                self.greed = 0
            elif self.greed > 100:
                self.greed = 100

            if debug:
                dbg("addGreed({}) = {:0.2f}".format(greed, self.greed))

        # Test greediness.
        #
        def isGreedy(self, value=0):
            if value == 0:
                return self.greed > 0
            return self.greed >= value

        # Earn money.
        #
        def earn(self, money):
            self.money += money

        # Can a cost be afforded?
        #
        def canAfford(self, money, acc=0):
            return money <= self.money + acc

        # Can a cost be afforded?
        #
        def canAffordScale(self, money, scale, acc=0):
            return money <= self.money * scale + acc

        # Can a food be afforded?
        #
        def canAffordFood(self, name, acc=0):
            tuple = food.getFood(name)
            if tuple is None:
                return false
            return tuple[3] + acc <= self.money

        # Can a food be afforded?
        #
        def canAffordFoodScale(self, name, scale, acc=0):
            tuple = food.getFood(name)
            if tuple is None:
                return false
            return tuple[3] * scale + acc <= self.money

        # Can a list of foods be afforded?
        #
        def canAffordFoodList(self, *names):
            acc = 0
            for name in names:
                tuple = food.getFood(name)
                if tuple is not None:
                    acc += tuple[3]

            return acc <= self.money

        # Spend money.
        #
        def spend(self, money):
            self.money -= money

            if self.money < 0:
                self.money = 0

        # Add a tip to a cost and round up to whole amount.
        # If tip is zero (default) round up to nearest quarter.
        #
        def roundUp(self, cost, tipPer=0):
            if tipPer == 0:
                result = math.ceil(cost * 4) / 4.0
            else:
                result = cost + (cost * tipPer) / 100.0
                result = math.ceil(result)

            # if debug:
            #    dbg("roundUp({}, {})={}".format(cost, tipPer, result))

            return result

        def roundDown(self, value):
            return math.floor(value)


        # Set mood to a specific value 0..100
        #
        def setMood(self, value):
            if value < 0:
                value = 0
            if value > 100:
                value = 100
            self.mood = value
            self.quantMood()

        # Set mood to the midpoint of a named mood.
        #
        def setMoodName(self, name):
            old = 0
            for mood in moods:
                # dbg("setMoodName: {} {}".format(mood[0], mood[1]))
                if name == mood[0]:
                    self.moodStr = name
                    self.mood = (old + mood[1])/2

                    if debug:
                        dbg("setMoodName({})={}".format(name, self.mood))

                    return
                else:
                    old = mood[1]

            dbg("setMoodName({}) mood not known".format(name))
            return

        # Add to mood value.
        #
        def addMood(self, value):
            self.mood += value
            if self.mood < 0:
                self.mood = 0
            if self.mood > 100:
                self.mood = 100
            self.quantMood()

            if debug:
                dbg("addMood({})={},{}".format(value, self.mood, self.moodStr))

        # Quantify the mood value to the mood string.
        #
        def quantMood(self):
            for mood in moods:
                # dbg("quantMood: {} {}".format(mood[0], mood[1]))
                if self.mood < mood[1]:
                    self.moodStr = mood[0]
                    break
            if debugM:
                dbg("quantMood()={}".format(self.moodStr))


        # Adjust prompt based on time of day and meals eaten.
        #
        # mealsize is passed in to avoid multiple evaluations.
        # Using mealsize=1 will avoid mention of hunger.
        # Using hour of zero will avoid mentioning meal times.
        #
        def appendPrompt(self, prompt, hour, mealsize):
            if hour >= 12 and hour < 16 and self.meals < 2:
                if mealsize <= 0:
                    prompt += " You aren't really hungry, but it is lunchtime."
                elif mealsize <= 4:
                    prompt += " Some lunch might be a good idea."
                elif mealsize <= 8:
                    prompt += " It's lunchtime, and you are hungry."
                else:
                    prompt += " It's lunchtime, and you are really hungry."
            elif hour >= 18 and hour <= 23 and self.meals < 3:
                if mealsize <= 0:
                    prompt += " You aren't really hungry, but it is dinnertime."
                elif mealsize <= 4:
                    prompt += " Some dinner might be a good idea."
                elif mealsize <= 8:
                    prompt += " It's dinnertime, and you are hungry."
                else:
                    prompt += " It's dinnertime, and you are really hungry."
            elif mealsize <= 0:
                "You aren't actually hungry."
            elif mealsize <= 4:
                "You could manage a bite to eat."
            elif mealsize <= 8:
                "You could certainly eat."
            elif mealsize <= 12:
                "You really want to eat."
            elif mealsize <= 16:
                "You really want to stuff yourself."
            else:
                "You could eat a horse."

            if self.cheatDay:
                prompt += " And it is, officially, a cheat day."
            elif self.diet:
                if self.dietWill >= 75:
                    prompt += " You are on a diet."
                elif self.dietWill >= 50:
                    prompt += " You are trying to diet."
                elif self.dietWill >= 25:
                    prompt += " You are sort of trying to diet."
                else:
                    prompt += " Your resolve to diet is fading."

            if mealsize > 10:
                prompt += " Your tummy growls."
            elif mealsize > 4:
                prompt += " Your tummy rumbles."

            return prompt

        # Shortcut for renpy.random.randint with debug
        #
        def random(self, low, high):
            result = renpy.random.randint(low, high)

            if debugM:
                dbg("random({},{})={}".format(low, high, result))

            return result

        # Random chance, higher percentages (0.0 .. 100.0) result in more True results.
        # A percent of zero or less is always false.
        # A percent of 100 or more is always true.
        #
        def randomBool(self, percent):
            if percent <= 0.0:
                result = False
            elif percent >= 100.0:
                result = True
            else:
                result = renpy.random.random() <= (percent / 100.0)

            if debugM:
                dbg("randomBool({:0.1f})={}".format(percent, result))

            return result

        # Random chance of at least min, but rising to max with max greed.
        #
        # min   - minimum percentage 0..100
        # max   - maximum percentage 0..100
        #
        def randomGreed(self, min, max):
            chance = min + (max - min) * self.greed / 100.0
            result = pc.randomBool(chance)

            if debug:
                dbg("randomGreed({},{})={} chance={:0.1f}".format(min, max, result, chance))
            return result

        # Random chance of at least min, but rising to max with max body weight.
        #
        # min   - minimum percentage 0..100
        # max   - maximum percentage 0..100
        #
        def randomWeight(self, min, max):
            wf = 100 * self.weightKg / 750.0
            if wf > 100:
                wf = 100
            chance = min + (max - min) * wf / 100.0
            result = pc.randomBool(chance)

            if debug:
                dbg("randomWeight({},{})={} chance={:0.1f}".format(min, max, result, chance))
            return result

define moods = [
        ('miserable', 15),
        ('unhappy', 30),
        ('meh', 45),
        ('okay', 60),
        ('fair', 75),
        ('happy', 90),
        ('blissful', 101)
    ]
# EoF
