#
# Game world.
#
init python:

    # The world class drives the main game loop.
    #
    # Provides methods for working out which renpy labels to call.
    # See placeIf.rpy
    #
    class World:
        homeLabel = "nwGas"     # Where the pc usually sleeps label
        locLabel = "nwGas"      # The pc current location label
        workLabel = "nwGas"     # Where the pc currently works label
        workDays = [0,1,2,3,4]  # The days the pc works
        workStartHr = 9         # The hour work starts
        workEndHr = 17          # The hour work ends
        wakeHr = 8              # The hour to wake up
        tmpHomeLabel = None     # Temporary home label
        tmpHomeLastDay = 0      # Temporary home last day number
        tmpWakeHr = None        # The temporary wake time, or None
        tmpWorkLabel = None     # Where the pc is temping label
        tmpWorkStartDay = 0     # Temporary work start day number
        tmpWorkLastDay = 0      # Temporary work last day number
        tmpWorkDays = None      # The days the pc is temping, or None
        tmpWorkStartHr = 0      # The temp work start hour
        tmpWorkEndHr = 0        # The temp work end hour

        working = False         # Is pc working
        workLastLabel = None    # Last work job
        dayOver = False         # True when end of day process begins
        noReturn = False        # True to prevent return of player to home
        noSleep = False         # True to prevent normal sleeping
        gameOver = False        # True on game over, exit main loop

        # Clear flags at start of day.
        #
        def newDay(self):
            self.working = False
            self.workLastLabel = None
            self.dayOver = False
            self.noReturn = False
            self.noSleep = False

        # Clear temporary work.
        #
        def clearTmpWork(self):
            self.tmpWorkLabel = None
            self.tmpWorkStartDay = 0
            self.tmpWorkLastDay = 0
            self.tmpWorkDays = None
            self.tmpWorkStartHr = 0
            self.tmpWorkEndHr = 0

        # Get the label used for player free choice.
        # - try locLabel.freeChoice
        # - use homeLabel.freeChoice
        #
        def getFreeChoiceLabel(self):
            result = self.locLabel + ".freeChoice"
            if not self.hasLabel(result):
                result = self.homeLabel + ".freeChoice"
            if debugM:
                dbg("World.getFreeChoiceLabel()={}".format(result))
            return result

        # Get the label when returned home.
        #
        def getRetHomeLabel(self):
            result = self.homeLabel + ".retHome"
            if not self.hasLabel(result):
                result = None
            if debugM:
                dbg("World.getRetHomeLabel()={}".format(result))
            return result

        # Get the label to return home.
        # - try .retHomeFromHere at current location
        # - try .retHomeToHere at home location
        # - None if neither exists
        #
        def getRetToHomeLabel(self):
            result = self.locLabel + ".retHomeFromHere"
            if not self.hasLabel(result):
                result = self.homeLabel + ".retHomeToHere"
                if not self.hasLabel(result):
                    result = None
            if debugM:
                dbg("World.getRetToHomeLabel()={}".format(result))
            return result

        # Get the label to call on sleep.
        #
        def getSleepLabel(self):
            result = self.locLabel + ".sleep"
            if not self.hasLabel(result):
                result = self.homeLabel + ".sleep"
            if debugM:
                dbg("World.getSleepLabel()={}".format(result))
            return result

        # Get the time to wake up.
        #
        def getWakeHour(self):
            if self.tmpWakeHr != None:
                return self.tmpWakeHr
            return self.wakeHr

        # Get the label to call on wake.
        #
        def getWakeLabel(self):
            result = self.locLabel + ".wake"
            if not self.hasLabel(result) and self.tmpHomeLabel != None:
                result = self.tmpHomeLabel + ".wakeTmp"
            if not self.hasLabel(result):
                result = self.homeLabel + ".wake"
            if debugM:
                dbg("World.getWakeLabel()={}".format(result))
            return result

        # Get the label to call for normal work.
        #
        def getWorkLabel(self):
            result = self.workLabel + ".work"
            self.workLastLabel = result
            if debugM:
                dbg("World.getWorkLabel()={}".format(result))
            return result

        # Get the label to call for end of work.
        #
        def getWorkEndLabel(self):
            result = None
            if self.workLastLabel != None:
                result = self.workLastLabel + "End"
                # dbg("World.getWorkEndLabel: trying {}".format(result))
                if not self.hasLabel(result):
                    result = None
            if debugM:
                dbg("World.getWorkEndLabel()={}".format(result))
            return result

        # Get the label to call for normal work.
        #
        def getTmpWorkLabel(self):
            result = None
            if self.tmpWorkLabel != None:
                result = self.tmpWorkLabel + ".workTmp"
                self.workLastLabel = result
            if debugM:
                dbg("World.getTmpWorkLabel()={}".format(result))
            return result

        # Does a lable exist?
        #
        def hasLabel(self, labelName):
            return renpy.has_label(labelName)

        # Is the pc at their home location?
        #
        def isHome(self):
            if self.tmpHomeLabel != None:
                return self.locLabel == self.tmpHomeLabel
            return self.locLabel == self.homeLabel

        # Is the current location the given one?
        #
        def isLoc(self, labelName):
            return self.locLabel == labelName

        # Does the current location begin with the given partial label name?
        #
        def isLocLike(self, labelStart):
            return self.locLabel.startswith(labelStart)

        # Is it a temporary work day?
        #
        def isTmpWorkDay(self):
            if self.tmpWorkLabel == None:
                return False
            dayNum = gt.dayNum
            if dayNum < self.tmpWorkStartDay:
                return False
            if self.tmpWorkLastDay != 0 and dayNum > self.tmpWorkLastDay:
                self.clearTmpWork()
                return False
            return self.tmpWorkDays != None and gt.dayOw in self.tmpWorkDays

        # Is it time for temporary work?
        #
        def isTmpWorkTime(self):
            return self.isTmpWorkDay() and gt.hour >= self.tmpWorkStartHr and gt.hour < self.tmpWorkEndHr

        # Is it a normal work day?
        #
        def isWorkDay(self):
            return gt.dayOw in self.workDays

        # Is the pc at their work location?
        #
        def isWorkLoc(self):
            if self.isTmpWorkDay():
                return self.locLabel == self.tmpWorkLabel
            return self.locLabel == self.workLabel

        # Is it time for normal work?
        #
        def isWorkTime(self):
            return self.isWorkDay() and gt.hour >= self.workStartHr and gt.hour < self.workEndHr

        # Set the player location to a new one.
        #
        def setLoc(self, loc):
            if self.hasLabel(loc):
                self.locLabel = loc
            else:
                dbg("World.setLoc({}) - unknown label".format(loc))

        # Set the player location to home.
        #
        def setLocHome(self):
            self.locLabel = self.homeLabel

        # Set the player location to their normal work location.
        #
        def setLocWork(self):
            self.locLabel = self.workLabel

        # Set temporary home.
        #
        def setTmpHome(self, homeLabel):
            self.tmpHomeLabel = homeLabel

        # Set up temporary work.
        #
        def setTmpWork(self, workLabel, startHr, endHr, days):
            self.tmpWorkLabel = workLabel
            self.tmpWorkStartHr = startHr
            self.tmpWorkEndHr = endHr
            self.tmpWorkDays = days
            daysStr = ""
            for dayOw in days:
                daysStr += str(dayOw) + " "
            if debug:
                dbg("World.setTmpWork({}, {}, {}, {})".format(workLabel, startHr, endHr, daysStr))

        # Set work location.
        #
        def setWork(self, workLabel):
            self.workLabel = workLabel
