#
# Main game loop.
#
label main:

label .newDay:
    if debug:
        dbg "main.newDay"

    # Start day.
    # Set up variables for the day ahead.
    #
    $ world.newDay()
    $ pc.newDay()
    $ gt.nextDay(world.getWakeHour())

    # If the pc is hard dieting, check to see if they are at the soft limit.
    #
    if pc.dietHard:
        if pc.weightKg <= softLimKg:
            # Soft limit reached, allow normal willpower to take over.
            #
            $ pc.dietHard = False;
            if debug:
                dbg "main.newDay: Soft limit reached, hard diet disabled"
        else:
            # Some assistance in losing weight while above soft limit.
            #
            $ pc.fat -= 0.4
            $ pc.addGreed(-0.2)
            $ pc.addLim(-0.1)
            $ pc.updateWeight()
    elif pc.diet:
        # Dieting: give an assist, gradually reduce greed and accepted fat limit.
        #
        $ pc.fat -= 0.2
        $ pc.addGreed(-0.1)
        $ pc.addLim(-0.05)
        $ pc.updateWeight()

    call sizeChange.check

    # Wake up.
    #
    call expression world.getWakeLabel()

label .loop:
    $ dbgLogDet("main.loop")

    # Return from main on game over.
    #
    if world.gameOver:
        return

    # Test for end of day conditions.
    #
    if not world.dayOver:
        if pc.energy == 0:
            call player.exhausted
            $ world.dayOver = True
        elif gt.hour < 8:
            call player.tooLate
            $ world.dayOver = True

    # Work, free choice, or events.
    #
    if not world.dayOver:
        if world.isTmpWorkTime():
            $ renLabel = world.getTmpWorkLabel()
            $ dbgLogDet("Main: Temporary work [renLabel]")
        elif world.isWorkTime():
            $ renLabel = world.getWorkLabel()
            $ dbgLogDet("Main: Work [renLabel]")
        elif world.working:
            # End of work day hook.
            #
            $ dbgLogDet("Main: Ending work")
            call .workEnd
            jump .loop
        else:
            # Not working.
            #
            $ renLabel = None
            $ dbgLogDet("Main: Free time")

        if renLabel != None:
            call util.checkEventOr(renLabel)
        else:
            call .freeOrEvent
        jump .loop

    #
    # Falling into...

    # Return home if required.
    #
label .endDayRet:
    # dbg "main.endDayRet"
    if not world.isHome() and not world.noReturn:
        $ renLabel = world.getRetToHomeLabel()
        if renLabel != None:
            call expression renLabel
        else:
            $ world.locLabel = world.homeLabel

        $ renLabel = world.getRetHomeLabel()
        if renLabel != None:
            call expression renLabel
    #
    # Falling into...

    # Go to sleep
    #
label .endDay:
    # dbg "main.endDay"
    # Sleep..
    #
    if not world.noSleep:
        call expression world.getSleepLabel()
    jump .newDay


    # -------------------------------------------------------------------------
    # Callable labels
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    # Either have an event of offer free choice.
    #
label .freeOrEvent:
    # dbg "main.freeOrEvent"
    $ renLabel = world.getFreeChoiceLabel()
    call util.checkEventOr(renLabel)
    return

    # Call the end of work hook.
    # Resets world.working.
    #
label .workEnd:
    if debugM:
        dbg "main.workEnd workLastLabel=[world.workLastLabel]"
    $ renpy.dynamic('workEndlabel')
    if world.workLastLabel != None:
        $ workEndlabel = world.getWorkEndLabel()
        # dbg "workEndLabel=[workEndlabel]"
        if workEndlabel != None:
            call expression workEndlabel
        $ world.workLastLabel = None
    $ world.working = False
    return
