﻿#
# Start script.
#
define dbg = Character("Debug")     # The voice of debug

# Init objects.
#
# Use default to mark for persistance in save games.
#
default eventList = []              # Scheduled event list
default gt = GameTime()             # Game date and time
default flags = Flags()             # General flags
default flagSet = set()             # Generic flag set
default daySet  = set()             # Day flag set
default dayFlags = Flags()          # Flags reset at day roll over
default countMap = dict()           # Generic counter map
default dayMap = dict()             # Day counter map
default counters = Counters()       # General purpose counters
default dayCounters = Counters()    # Counters reset each day
default pc = Player()               # The player
default pcClothes = Clothing()      # The player's clothes
default pcClothesMap = dict()       # The player's clothes map
default alicia = Npc()              # The owner of the clothes shop
default lacy = Npc()                # The waitress in the bar
default sofia = Npc()               # The waitress in the diner, love interest
default steve = Npc()               # The forward guy in the bar, love interest
default world = World()             # World controlling main loop
default foodMenu = FoodMenu()       # Generic food menu
default foodMenuMap = dict()        # Food menu map

default debug = False
default debugM = False
default debugShow = debug

# Init characters.
#
# Using character.xxx allows for a variable xxx to exist too.
#
define character.pc = Character("[pc.name]", color="#994444")
define character.alicia = Character("Alicia")       # Clothes shop owner
define character.barman = Character("Barman")       # Generic barman
define character.customer = Character("Customer")   # Generic customer
define character.dj = Character("DJ")               # Radio station DJ
define character.joe = Character("Joe")             # Garage owner
define character.lacy = Character("Lacy")           # Bar waitress
define character.sfx = Character("SFX:")            # Radio sound effects
define character.sofia = Character("Sofia")         # Diner waitress
define character.steve = Character("Steve")         # A friendly guy
define character.vendor = Character("Vendor")       # Generic vendor

# The game starts here.
#
label start:
    # Set up storage.
    #
    # $ debug = True
    $ gt.setEventList(eventList)
    $ flags.setSet(flagSet)
    $ dayFlags.setSet(daySet)
    $ counters.setMap(countMap)
    $ dayCounters.setMap(dayMap)
    $ foodMenu.setMap(foodMenuMap)
    $ pcClothes.setMap(pcClothesMap)
    $ lastBg = None
    $ showBg('dusty-road')

    # Require consent.
    #
    call .consent
    if not consent:
        return

    # Acknowledge warnings.
    #
    call .warnings
    if not warningPass:
        return

    # NPC prefs.
    #
    $ lacy.pcMinBfp = 30
    $ lacy.pcMaxBfp = 45
    $ sofia.pcMinBfp = 35
    $ sofia.pcMaxBfp = 64   # Sofia's own size-ish
    $ sofia.weight = 110    # 242lb
    $ steve.pcMaxBfp = 30
    $ steve.weight = 74     # 163lb

    # Show debug if enabled.
    #
    show screen stats
    if debugShow:
        show screen debugStats
        $ tmp = food.listFoods()
        dbg "[tmp]" (what_size=12)

    # Play the intro.
    call intro

    # Run the main game loop.
    call main
    return

    # Require the player to consent.
    #
label .consent:
    menu:
        "This game contains adult themes. Is this appropriate for you?"
        "Yes, I am over the age of 18, and over the age of consent where I live.":
            $ consent = True
        "I'm old enough, but don't want any of that":
            $ consent = False
            "
            This game is likely not for you then.
            "
            return
        "No, I'm not old enough":
            $ consent = False
            "
            Sorry, but you can't play then.
            "
            return

    menu:
        "And you're doing this on a computer where such activity is permitted?"
        "Yes, I'm in my room on my own machine and I won't be interrupted":
            $ pass
        "I'm at work":
            jump .iAmAnIdiot
        "I'm alone on my work laptop":
            jump .iAmAnIdiot
        "I'm on a school machine":
            jump .iAmAnIdiot
        "I'm in a public library":
            jump .iAmAnIdiot
        "I'm in a public place where anyone could read my screen":
            jump .iHaveNoShame
        "I'm conducting academic research into fetish games":
            "Really? Cool choice of subject matter expertise."
        "It's my job to review games":
            "In that case, enjoy!"
    return

    # Player has done something real stupid.
    #
label .iAmAnIdiot:
    "Not your best plan. Time to uninstall, securely delete the recycle bin,
     and clear your browser history."
    $ consent = False
    return

    # Player okay but has no shame.
    #
label .iHaveNoShame:
    "Just so long as you've thought about what you are about to do."
    "You could still just close the game."
    return

    # Show warnings and require appropriate responses.
    #
label .warnings:
    menu:
        "You will be playing a human female character, and exploring topics
        including your own weight gain, societal pressure, relationships,
        feeding, being fed."
        "That's what I'm here for!":
            "
            That's good to know. I think your tastes can be accommodated.
            "
            $ warningPass = True
        "Actually, no":
            "
            That's fine, but the game is likely not for you.
            "
            $ warningPass = False
            return

    menu:
        "This game is an experiment,
        is unfinished,
        contains no visuals,
        and may never be finished."
        "I'm okay with that, show me what there is":
            $ pass
        "I think I'll pass":
            "
            Okay, that's fine. At least it wasn't a huge download.
            "
            $ warningPass = False
            return

    menu:
        "Finally, the game contains a debugger, primarily for development use.
        Using the debugger may spoil some things,
        allow you to break the game,
        reveal hidden variables,
        and basically cheat."
        "Let me pop the hood and get my hands dirty":
            call .debugHelp
            $ debugShow = True
            $ debug = True
            $ debugM = False
        "I am pure of heart and reject these devilish tools":
            "Yours is a noble calling.
            I hope you are as good at resisting other temptations!"
            $ debugShow = False
            $ debug = False
            $ debugM = False
    return

    # Debugger notice.
    #
label .debugHelp:
    "{size=+8}Warranty Void{/size}"
    """
    Click on the {b}Debug open{/b} in the top left to show debug variables.
    Click on {b}Debug close{/b} to collapse it.

    With the debug variables open clicking on {b}Inline debug{/b}
    will cycle through inline debug detail settings.

    Periodically during play a debug menu will appear.
    To advance without changing anything, just choose
    \"Return to game\".
    """
    return

# EoF
