#
# Tests for size change.
#
init python:

    # Size limits for pc based on fat in kgs.
    # Base on 40kg base weight.
    #
    #       mass kg     weight lb   bfp     notes
    #  0:    40.. 45     88.. 99    < 11.1  A skinny
    #  1:    45.. 50     99..110    < 20.0  B thin
    #  2:    50.. 60    110..132    < 33.3  C normal
    #  3:    60.. 70    132..154    < 42.9  D plump
    #  4:    70.. 80    154..176    < 50.0  E fat
    #  5:    80..108    176..237    < 63.0  F obese
    #  6:   108..136    237..300    < 70.7  G (near sofia)
    #  7:   137..182    300..400    < 78.0  H bbw
    #  8:   182..227    400..500    < 82.4  I
    #  9:   227..273    500..600    < 85.3  J
    # 10:   273..318    600..700    < 87.4  K ssbbw
    # 11:   318..364+   700..800+           L
    #
    sizeLims = (5, 10, 20, 30, 40, 68, 96.4, 142, 187.3, 233, 278.3)

    # Look up fat in kg against size limits.
    # Returns size number
    #
    def sizeLookup(fat):
        idx = 0;
        for lim in sizeLims:
            if fat < lim:
                return idx
            else:
                idx += 1
        return idx

    # How far along a size rangw is a given fat weight?
    # Result is a percentage 0..100.
    #
    def sizeProgress(fatKg, gsize):
        idx = 0
        lowKg = 0
        if gsize > 0:
            lowKg = sizeLims[gsize-1]
        if gsize == 11:
            maxKg = 324
        else:
            maxKg = sizeLims[gsize]
        return max(0,min(100,int(round(100*(fatKg-lowKg)/(maxKg-lowKg)))))


label sizeChange:

    # Check and flag size changes.
    # Updates pc.gsize
    #
label .check:
    $ nsize = sizeLookup(pc.fat)
    $ osize = pc.gsize
    if nsize == osize:
        if debug:
            dbg "check: size unchanged"
        return

    $ pc.gsize = nsize

    if nsize > osize:
        $ flags.set("sizeU")
        $ flags.clear("sizeD")
        if debug:
            dbg "check: size increase to [nsize]"
    else:
        $ flags.set("sizeD")
        $ flags.clear("sizeU")
        if debug:
            dbg "check: size decrease to [nsize]"
    return
