
init python:

    # Cardinal number words.
    #
    cardinals = [
        'zero',
        'one',
        'two',
        'three',
        'four',
        'five',
        'six',
        'seven',
        'eight',
        'nine',
        'ten',
        'eleven',
        'twelve',
        'thirteen',
        'fourteen',
        'fifteen',
        'sixteen',
        'seventeen',
        'eighteen',
        'nineteen',
        'twenty'
    ]

    # Ordinal number words.
    #
    ordinals = [
        'first',
        'second',
        'third',
        'fourth',
        'fifth',
        'sixth',
        'seventh',
        'eighth',
        'nineth',
        'tenth',
        'eleventh',
        'twelvth',
        'thirteenth',
        'fourteenth',
        'fifteenth',
        'sixteenth',
        'seventeenth',
        'eighteenth',
        'nineteenth',
        'twentieth'
    ]


    # Get the ordinal for a value: first, second, third ...
    #
    def ordinal(value):
        if value == 0:
            return '0th'
        idx = value - 1
        if idx < 0:
            return '-ve'
        if idx < len(ordinals):
            return ordinals[idx]
        idx = value % 10
        if idx == 2:
            return str(value)+'nd'
        if idx == 3:
            return str(value)+'rd'
        return str(value)+'th'

    # Get the cardinal for a value: one, two, three ...
    #
    def cardinal(value):
        if value >= len(cardinals):
            return str(value)
        return cardinals[value]

    # Get clock time as words.
    #
    def clockTime(hour):
        if hour <= 8:
            return cardinal(hour) + " in the morning"
        elif hour <= 12:
            return cardinal(hour)
        elif hour <= 20:
            return cardinal(hour % 12)
        else:
            return cardinal(hour % 12) + " at night"

    # Convert a number of days into a rough time period string.
    # Tends to round uo.
    #
    def roughPeriod(days):
        if days == 1:
            period = "only a day"
        elif days == 7:
            period = "a week"
        elif days == 14:
            period = "a couple of weeks"
        elif days == 30:
            period = "a month"
        elif days == 365:
            period = "a year"
        if days < 60:
            period = "{} days".format(int(days))
        elif days < 24 * 30:
            period = "{} months".format(int(math.ceil(days/30.0)))
        else:
            period = "{} years".format(int(math.ceil(days/365.0)))
        return period

    # Is a string/unicode None or empty?
    #
    def isEmpty(str):
        if str == None:
            return True
        return len(str) == 0

    # Toggle whether a specific day of week is in a list.
    #
    def toggleDayOw(dayList, dayOw):
        if dayOw in dayList:
            dayList.remove(dayOw)
        else:
            dayList.append(dayOw)

    # Does an image file exist?
    # Looks for .png or .jpg.
    #
    def existsImg(filename):
        result = renpy.loadable(filename + '.png')
        if not result:
            result = renpy.loadable(filename + '.jpg')
        if debugM:
            dbg("existsImg({})={}".format(filename, result))
        return result

    # Get a background image file.
    # If shot is not None/empty search:
    # - loc-shot-gsize
    # - loc-shot
    # - shot
    # Else search:
    # - loc-gsize
    # - loc
    #
    def getBgFile(shot, location=None):
        # dbg("getBgFile({},{})".format(shot, location))
        if location == None:
            location = world.locLabel
        gsize = pc.gsize
        if isEmpty(shot):
            candidate = "{}-{}".format(location, gsize)
            # dbg("trying {}".format(candidate))
            if not existsImg(candidate):
                candidate = "{}".format(location)
        else:
            candidate = "{}-{}-{}".format(location, shot, gsize)
            # dbg("trying {}".format(candidate))
            if not existsImg(candidate):
                candidate = "{}-{}".format(location, shot)
                # dbg("trying {}".format(candidate))
                if not existsImg(candidate):
                    candidate = "{}".format(shot)
        # dbg("trying {}".format(candidate))
        if not existsImg(candidate):
            candidate = None
            if debug:
                dbg("getBgFile({},{}) gsize={} could not find any file".format(shot, location, gsize))
        elif debug:
            dbg("getBgFile({},{})={} gsize={}".format(shot, location, candidate, gsize))
        return candidate

    # Show a new scene with a background image.
    # See getBgFile.
    # Sets global lastBg.
    #
    def showBg(shot, location=None):
        global lastBg
        bgFile = getBgFile(shot, location)
        if bgFile == None:
            bgFile = "blank"
        # dbg("Using '{}'".format(bgFile))
        renpy.scene()
        renpy.show(bgFile)
        lastBg = bgFile

    # Restore a new scene with a background image.
    #
    def restoreBg(bgFile):
        global lastBg
        renpy.scene()
        renpy.show(bgFile)
        lastBg = bgFile

    # Have dbg say some text if debug is on.
    #
    def dbgLog(text):
        if debug:
            dbg(text)

    # Have dbg say some text if debug detail is on.
    #
    def dbgLogDet(text):
        if debugM:
            dbg(text)

    # Calculate how much body fat is burned per day by a female.
    #
    def bmrFatFem(heightCm, weightKg, age):
        kcal = 9.56 * weightKg
        kcal += 1.85 * heightCm
        kcal -= 4.67 * age
        kcal += 655
        mass = kcal / 7770
        return mass


    # Utility renpy routines
    #
label util:

    # Check for a scheduled event, and either call it directly
    # or offer it as a choice if it has a non-None description.
    # If the event is rejected and it has a non-None rejLabel call that.
    #
    # Call the provided alt label if there is no event or it is rejected.
    # The alt label must be specified in full ie. global.local
    #
label .checkEventOr(altLabel=None):
    if altLabel == None:
        dbg "Util.checkEventOr called with no altLabel defined."
        dbg "[RhubarbRhubarbRhubarb]"      # Need to cause an error for a stack trace

    $ sched = gt.getEventNow()
    if sched != None:
        if sched.desc != None:
            # If there's a description, present a choice.
            #
            menu:
                "[sched.desc] Do you still want to go?"
                "Yes":
                    call expression sched.evLabel
                "No":
                    # If there's a rejection label, call it.
                    #
                    if sched.rejLabel != None:
                        call expression sched.rejLabel
                    else:
                        "I've changed my mind."
                    call expression altLabel
        else:
            call expression sched.evLabel
        $ gt.doneEvent(sched)
    else:
        call expression altLabel
    return
